MODULE module_netcdf
   
  CONTAINS
  
  !-----------------------------------------------------------------------------
  
  SUBROUTINE write_netcdf(archive,var_name,x_n,y_n,z_n,data_xyz)
    
    USE netcdf
    
    IMPLICIT NONE
    
    INTEGER, PARAMETER :: dp = kind(0.0D0)
    INTEGER, PARAMETER :: strlen = 32
    
    ! I/O
    
    INTEGER :: x_n,y_n,z_n
    
    CHARACTER (LEN=strlen) :: archive,var_name
    
    REAL(dp), DIMENSION(:,:,:), ALLOCATABLE :: data_xyz
    
    ! Local variables
    
    INTEGER :: nc_id, var_id, status
    
    INTEGER :: x_dim_id,y_dim_id,z_dim_id
    
    INTEGER :: dim_ids(3)
    
    ! Do your worst
    
    WRITE(*,*) 'Writing netCDF archive ',archive
    
    status = nf90_create(archive, NF90_CLOBBER, nc_id)
    
    status = nf90_def_dim(nc_id, 'x', x_n, x_dim_id)
    status = nf90_def_dim(nc_id, 'y', y_n, y_dim_id)
    status = nf90_def_dim(nc_id, 'z', z_n, z_dim_id)
    
    dim_ids = (/ x_dim_id,y_dim_id,z_dim_id /)
    
    ! Write a variable
    
    status = nf90_def_var(nc_id, var_name, NF90_DOUBLE, dim_ids, var_id)
    
    status = nf90_enddef(nc_id)
    
    ! Write the data
    
    status = nf90_open(archive,nf90_write,nc_id)
    
    status = nf90_inq_varid(nc_id,var_name,var_id)
    
    status = nf90_put_var(nc_id, var_id, data_xyz, start = (/ 1,1,1 /), count = (/ x_n,y_n,z_n /))
    
    status = nf90_close(nc_id)
    
  END SUBROUTINE write_netcdf
  
  !-----------------------------------------------------------------------------
  
  SUBROUTINE read_netcdf(archive,var_name,x_n,y_n,z_n,data_xyz)
    
    USE netcdf
    
    IMPLICIT NONE
    
    INTEGER, PARAMETER :: dp = kind(0.0D0)
    INTEGER, PARAMETER :: strlen = 32
    
    ! I/O
    
    INTEGER :: x_n,y_n,z_n
    
    CHARACTER (LEN=strlen) :: archive,var_name
    
    REAL(dp), DIMENSION(:,:,:), ALLOCATABLE :: data_xyz
    
    ! Local variables
    
    INTEGER :: nc_id, var_id, status
    
    ! Do your worst
    
    WRITE(*,*) 'Reading netCDF archive ',archive
    
    status = nf90_open(archive,nf90_nowrite,nc_id)
    
    status = nf90_inq_varid(nc_id,var_name,var_id)
    
    status = nf90_get_var(nc_id, var_id, data_xyz, start = (/ 1,1,1 /), count = (/ x_n,y_n,z_n /))
    
    status = nf90_close(nc_id)
    
  END SUBROUTINE read_netcdf
  
END MODULE module_netcdf
